(function () {
    'use strict';

    app.routers.App = Backbone.Router.extend({
        firstPage: true,
        routes: {
            '': 'homepage',
            'error':'error',
            'about': 'about',
            'engains': 'engains',
            'favorites': 'favorites',
            'propositions': 'propositions',
            'results': 'results',
            'tickets': 'tickets'
        },
        currentPage: null,
        previousPage: null,
        before: function () {
            $.mobile.loading('show');
            app.helpers.removeAds();
        },
        after: function () {
            if (this.currentPage.sync) {
                this.changePage();
            }
            app.helpers.addAds();
        },
        changePage: function () {
            var transition = (this.firstPage) ? 'none' : 'slide';
            var reverse = app.views.app.reverse;

            if (!app.views[this.currentPage]) {
                this.currentPage.render();
                $('body').append($(this.currentPage.el));
            }

            $.mobile.changePage($(this.currentPage.el), { changeHash: false, transition: transition, reverse: reverse, showLoadMsg: true}, true, true);
            $.mobile.loading('hide');
        },
        initialize: function () {
            var _self = this;

            

            this.route(/^results\/(\d+)\/(\d+)\/(\d+)$/, "results");

            this.on('route', function (r) {
                
            });

            // Instantiate app model
            app.models.application = new app.models.Application();

            // Instantiate app view
            app.views.app = new app.views.App({
                transition: 'none'
            });

            app.collections.favorites = new app.collections.Favorites();

            $(document).on("pageshow", function (event, ui) {

                if (!_self.firstPage && _self.previousPage) {
                    _self.previousPage.close();
                    _self.previousPage = null;
                    _self.previousPage = _self.currentPage;
                    ui.prevPage.off().unbind().remove();
                } else {
                    _self.previousPage = _self.currentPage;
                }

                _self.firstPage = false;
            });

            Backbone.on('changePage', function () {
                _self.changePage();
            });

            $(document).ajaxError(function (e, xhr, options) {
                Backbone.history.navigate('#error', {trigger: true, silent: false});
            });

        },
        about: function () {
            this.currentPage = app.views.about = new app.views.About();
        },
        engains: function () {
            this.currentPage = app.views.engains = new app.views.Engains({model: app.models.application});
            app.models.application.setUrl();
            app.models.application.fetch();
        },
        homepage: function () {
            this.currentPage = app.views.homepage = new app.views.HomePage();
        },
        error: function () {
            this.currentPage = app.views.error = new app.views.Error();
        },
        favorites: function () {

            this.currentPage = app.views.favorites = new app.views.Favorites({
                collection: app.collections.favorites
            });
            app.collections.favorites.fetch({ reset: true});

        },
        propositions: function (category) {

            // Instanciate new collection of propositions
            app.collections.propositions = new app.collections.Propositions();

            // Instanciate new view for propostions list
            this.currentPage = app.views.propositions = new app.views.Propositions({
                collection: app.collections.propositions,
                favorites: app.collections.favorites,
                category: category,
                model: app.models.application
            });

            app.collections.propositions.fetch({reset: true});
            this.currentPage.model.setUrl();
            this.currentPage.model.fetch();


        },
        results: function (day, month, year) {
            var page;

            if (day && month && year) {
                page = 'resultspage';
            } else {
                page = 'resultscategory';
            }

            // Instanciate new collection of propositions
            if (!app.collections.results) {
                app.collections.results = new app.collections.Results();
            }
            app.collections.results.day = day;
            app.collections.results.month = month;
            app.collections.results.year = year;

            this.currentPage = app.views.results = new app.views.Results({
                collection: app.collections.results,
                favorites: app.collections.favorites,
                model: app.models.application,
                pageName: page
            });
            this.currentPage.model.day = day;
            this.currentPage.model.month = month;
            this.currentPage.model.year = year;


            app.collections.results.setUrl();
            app.collections.results.fetch({reset: true});
            this.currentPage.model.setUrl();
            this.currentPage.model.fetch();


        },
        tickets: function () {
            if (!app.models.tickets) {
                app.models.tickets = new app.models.Tickets();
            }
            this.currentPage = app.views.tickets = new app.views.Tickets({model: app.models.tickets});
            this.currentPage.model.fetch();
        }
    });

})();