(function () {
    'use strict';

    app.views.Favorites = app.views.Page.extend({
        pageName: 'favorites',
        category: 'all',
        list: [],
        sync: true,
        initialize: function (options) {
            var that;

            

            that = this;

            this.category = options.category || this.category;

            // Extend events from parent
            _.extend(this.events, this.constructor.__super__.events);

            this._super();

        },
        render: function () {

            this._super();

            this.renderAll();
            this.renderFavorites();
            this._showCategory();

            return this;
        },
        renderFavorite: function (favorite, $elm) {
            
            this.addOne(favorite, $elm, 'favorites');
        },
        renderFavorites: function () {
            var that;

            that = this;

            _.each(this.collection.getFavorites(), function (favorite) {
                that.renderFavorite(favorite, this.$('#favorites-favorites'));
            }, this);
        },
        removeFavorite: function (view) {

            view.unbind();
            view.stopListening();
            view.remove();

            this.list = _.reject(this.list, function (v) {
                return v.cid === view.cid;
            });

        },
        refreshState: function (view) {
            view.$el.find('option[value="false"]').attr('selected', 'selected');
            view.$el.find('select[data-role="slider"]').slider('refresh');
        },
        addOne: function (favorite, $elm, category) {
            var view, $el, className;

            className = (category === 'all') ? 'has-filter' : '';

            view = new app.views.Favorite({collection: this.collection, model: favorite, category: category, className: className}),
                $el = $(view.render().el);

            $elm = $elm || this.$('#favorites-' + favorite.get('cat'));


            $elm.append(view.render().el);
            app.views.favorites.list.push(view);


            $el.find('select[data-role="slider"]').slider();
        },
        renderAll: function () {
            var collection, that;

            that = this,
                collection = this.collection;

            _.each(collection.getModule(collection.modules.CDC), function (favorite) {
                
                that.addOne(favorite, null, 'all');
            }, this);
            _.each(collection.getModule(collection.modules.ROQUEPINE),
                function (favorite) {
                    that.addOne(favorite, null, 'all');
                }, this);
            _.each(collection.getModule(collection.modules.QUOVADIS),
                function (favorite) {
                    that.addOne(favorite, null, 'all');
                }, this);
            _.each(collection.getModule(collection.modules.SASSAFRAS), function (favorite) {
                that.addOne(favorite, null, 'all');
            }, this);
            _.each(collection.getModule(collection.modules.ASTRES), function (favorite) {
                that.addOne(favorite, null, 'all');
            }, this);

        }
    });


    app.views.Favorite = app.views.Favorites.extend({
        category: '',
        initialize: function (options) {
            

            var that;

            that = this;

            this.category = options.category;
            this.className = options.className || this.className;

            this.template = app.helpers.template('favorite', 'favoritetpl', this.model.toJSON());

            this.listenTo(this.model, 'change', function (model) { // add is buggy with localstorage plugin
                if (model.get('fav') && that.category === 'all') {
                    that.renderFavorite(model, $('#favorites-favorites'));
                }
                else if (that.category === 'favorites') {
                    that.removeFavorite(that);
                } else {
                    that.refreshState(this);
                }
            });


        },
        events: {
            'change select': 'changeFavorite'
        },
        tagName: 'tr',
        className: '',
        render: function () {
            this.$el.html(this.template);
            if (this.model.get('fav')) {
                this.$el.find('option[value="true"]').attr('selected', 'selected');
            }
            return this;
        },
        changeFavorite: function (e) {
            var value, item;

            value = e.target.value === 'true' ? true : false,
                item = this.collection.get(this.model.cid);

            if (value) {
                this.collection.create({id: this.model.get('id')});
                this.model.set({fav: true});
            } else if (!value && this.category === 'favorites') {
                this.model.set({fav: false});
                this.collection.remove(item);
            } else {
                this.model.set({fav: false});
                this.collection.remove(item);
            }
        }
    });

}());
