(function () {
    'use strict';

    app.views.Page = Backbone.View.extend({
        data: {},
        fetched: 0,
        transition: 'none',
        navTpl: '',
        panelNavTpl: '',
        infosTpl: '',
        events: {
            'click a': 'setTransition',
            'click #btn-nav': 'openNav',
            'click .js-btn-reload': 'reloadPage',
            'click #filters a': 'changeCategory',
            'click .js-open-url': 'openUrl'

        },
        initialize: function (options) {

            var that;

            

            that = this, options = options || {};

            this.navTpl = app.helpers.template('_nav', '_navtpl');
            this.panelNavTpl = app.helpers.template('_panelnav', '_panelnavtpl', {
                nav: this.navTpl
            });

            this.data = {
                nav: this.navTpl,
                panelNav: this.panelNavTpl,
                date: this.getDate()
            };

            _.extend(this.data, options.data, options.data || {});


        },
        render: function () {
            if (this.data.error) {
                this.data.error += '<br><button class="js-btn-reload">Réessayer</button>';
            }
            this.template = app.helpers.template(this.pageName, this.pageName + 'tpl', this.data);
            this.$el.addClass('page-' + this.pageName).attr('id', 'page-' + this.pageName).attr('data-role', 'page').append(this.template);
            this.$el.find('table').addClass('ui-responsive table-stroke ui-table');
            return this;
        },
        setTransition: function () {
            app.views.app.transition = 'slide';
        },
        openNav: function () {
            $("#panel-nav").panel("open");
        },
        _hasFetchedAll: function () {
            return this.async === this.fetched;
        },
        changePage: function () {
            this.fetched++;
            if (this._hasFetchedAll()) {
                Backbone.trigger('changePage');
            }
        },
        reloadPage: function () {
            window.location.reload();
        },
        _showCategory: function () {
            this.$el.find('.is-visible').removeClass('is-visible');
            this.$el.find('#filters .ui-btn-active').removeClass('ui-btn-active');
            this.$el.find('#' + this.category + '-part').addClass('is-visible');
            this.$el.find('#filter-' + this.category).addClass('ui-btn-active');
        },
        changeCategory: function (e) {

            e.preventDefault();

            var category;

            category = e.currentTarget.href.split('#')[1];

            app.helpers.removeAds();

            if (category === 'random') {
                this.renderRandom();
            }

            this.category = category;

            this._showCategory();

            app.helpers.addAds();

        },
        getDate: function () {
            if (this.collection && this.collection.day) {
                return this.collection.day + '/' + this.collection.month + '/' + this.collection.year;
            } else {
                var d;
                d = new Date();
                return [(d.getDate() < 10 ? '0' + d.getDate() : d.getDate()), (d.getMonth() + 1 < 10 ? '0' + (d.getMonth() + 1) : d.getMonth() + 1), d.getFullYear()].join('/');
            }
        },
        openUrl: function (e) {
            e.preventDefault();
            app.helpers.openUrl(e.currentTarget.href);
        }

    });
}());