(function () {
    'use strict';

    app.views.Propositions = app.views.Page.extend({
        pageName: 'propositions',
        list: [],
        async: 3,
        category: 'all',
        favorites: null,
        initialize: function (options) {

            var that;

            that = this;

            this.category = options.category || this.category;
            this.favorites = options.favorites;

            this.listenTo(this.model, 'sync', function (model, data) {
                that.infosTpl = app.helpers.template('_infos', '_infostpl', data);
                _.extend(that.data, {
                    infos: that.infosTpl
                });
                that.changePage();
            });
            this.listenTo(this.favorites, 'reset', function () {
                this.data.favorites = this.favorites.getFavorites();
                that.changePage();
            });
            this.listenTo(this.collection, 'reset', function () {
                this.data.propositions = this.collection.models.length;
                this.data.error = this.collection.isInError();
                this.favorites.fetch({reset:true});
                that.changePage();
            });


            // Extend events from parent
            _.extend(this.events, this.constructor.__super__.events);
            this._super();

        },
        events: {
            'click #btn-add-favorites': 'addFavorites'
        },
        renderFavorites: function () {
            var that;
            that = this;
            
            _.each(this.data.favorites, function (favorite) {
                that.renderFavorite(favorite);
            }, this);
        },
        renderFavorite: function (favorite) {
            this.addOne(favorite, this.$('#propositions-favorites'), 'favorites');
        },
        renderAll: function () {
            var collection, that;

            that = this,
                collection = this.collection;

            _.each(collection.getModule(collection.modules.CDC), function (proposition) {
                that.addOne(proposition, null, 'all');
            }, this);
            _.each(collection.getModule(collection.modules.ROQUEPINE),
                function (proposition) {
                    that.addOne(proposition, null, 'all');
                }, this);
            _.each(collection.getModule(collection.modules.QUOVADIS),
                function (proposition) {
                    that.addOne(proposition, null, 'all');
                }, this);
            _.each(collection.getModule(collection.modules.SASSAFRAS), function (proposition) {
                that.addOne(proposition, null, 'all');
            }, this);
            _.each(collection.getModule(collection.modules.ASTRES), function (proposition) {
                that.addOne(proposition, null, 'all');
            }, this);

        },
        renderRandom: function () {
            var random, view;

            random = _.random(this.collection.models.length),
                view = app.views.propositions.list[random];

            this.$('#propositions-random').html(view.render().el);
            this.$('#propositions-random').find('tr').removeAttr('class');

        },
        addOne: function (proposition, $elm, category) {
            var view, $el, className;

            className = (category === 'all') ? 'has-filter' : '';

            view = new app.views.Proposition({model: proposition, category: category, className: className}),
                $el = $(view.render().el);

            $elm = $elm || this.$('#propositions-' + proposition.get('cat'));

            $elm.append(view.render().el);
            this.list.push(view);

        },
        render: function () {

            this._super();

            if (!this.data.error) {
                this.renderAll();
                this.renderFavorites();
                this.renderRandom();
                this._showCategory();
            }
            return this;
        },
        addFavorites: function () {
            Backbone.history.navigate('#favorites', {trigger: true, silent: false});
        }
    });


    app.views.Proposition = Backbone.View.extend({
        initialize: function (options) {
            
            this.template = app.helpers.template('proposition', 'propositiontpl', this.model.toJSON());

            this.className = options.className || this.className;
        },
        tagName: 'tr',
        className: '',
        render: function () {
            this.$el.html(this.template);
            return this;
        }
    });


}());
