var iabRef; //temp

(function () {
    'use strict';

    app.views.Results = app.views.Page.extend({
        pageName: '',
        category: 'all',
        favorites: null,
        async: 3,
        list: [],
        initialize: function (options) {
            

            var that;

            that = this;

            this.category = options.category || this.category;
            this.favorites = options.favorites;
            this.pageName = options.pageName;

            if (this.pageName === 'resultspage') {

                this.listenTo(this.model, 'sync', function (model, data) {
                    that.infosTpl = app.helpers.template('_infos', '_infostpl', data);
                    _.extend(that.data, {
                        infos: that.infosTpl
                    });

                    that.changePage();
                });
                this.listenTo(this.favorites, 'reset', function () {
                    this.data.favorites = this.favorites.getFavorites();
                    that.changePage();
                });
                this.listenTo(this.collection, 'reset', function () {
                    this.data.results = this.collection.models.length;
                    this.data.error = this.collection.isInError();
                    this.data.winners = this.collection.getWinners();
                    this.favorites.fetch({reset: true});
                    that.changePage();
                });


                this.data.favorites = this.favorites.getFavorites();


            } else {
                this.sync = true;
            }


            // Extend events from parent
            _.extend(this.events, this.constructor.__super__.events);

            this._super();
        },
        events: {
            'click #btn-back': 'changeDay',
            'click #btn-add-favorites': 'addFavorites'
        },
        addOne: function (result, $elm, category) {
            var view, $el, className;

            className = (category === 'all') ? 'has-filter' : '';

            view = new app.views.Result({model: result, category: category, className: className}),
                $el = $(view.render().el);

            $elm = $elm || this.$('#results-' + result.get('cat'));

            $elm.append(view.render().el);
            this.list.push(view);

        },
        renderAll: function () {
            var collection, that;

            that = this,
                collection = this.collection;


            _.each(collection.getModule(collection.modules.CDC), function (result) {
                that.addOne(result, null, 'all');
            }, this);
            _.each(collection.getModule(collection.modules.ROQUEPINE),
                function (result) {
                    that.addOne(result, null, 'all');
                }, this);
            _.each(collection.getModule(collection.modules.QUOVADIS),
                function (result) {
                    that.addOne(result, null, 'all');
                }, this);
            _.each(collection.getModule(collection.modules.SASSAFRAS), function (result) {
                that.addOne(result, null, 'all');
            }, this);
            _.each(collection.getModule(collection.modules.ASTRES), function (result) {
                that.addOne(result, null, 'all');
            }, this);

        },
        renderFavorites: function () {
            var that;
            that = this;
            _.each(this.data.favorites, function (favorite) {
                that.renderFavorite(favorite, this.$('#results-favorites'));
            }, this);
        },
        renderFavorite: function (favorite, $elm) {
            this.addOne(favorite, $elm, 'favorites');
        },
        renderWinners: function () {
            var that;


            that = this;

            _.each(this.data.winners, function (winner) {
                that.renderWinner(winner);
            }, this);


        },
        renderWinner: function (proposition) {
            this.addOne(proposition, this.$('#results-winners'), 'winners');
        },
        render: function () {
            this._super();
            if (this.pageName === 'resultscategory') {
                this.renderCalendar();
            } else {

                if (this.data.error) {
                    return this;
                }

                this.renderAll();
                this.renderFavorites();
                this.renderWinners();
                this._showCategory();
            }
            return this;
        },
        renderCalendar: function () {
            var $clndr = this.$('#mini-clndr');

            moment.lang('en', {
                months: [
                    'Janvier',
                    'Février',
                    'Mars',
                    'Avril',
                    'Mai',
                    'Juin',
                    'Juillet',
                    'Août',
                    'Septembre',
                    'Octobre',
                    'Novembre',
                    'Décembre'
                ]
            });


            // there are a lot of options. the rabbit hole is deep.
            $clndr.clndr({
                daysOfTheWeek: ['D', 'L', 'M', 'M', 'J', 'V', 'S'],
                //daysOfTheWeek: ['S', 'M', 'T', 'W', 'T', 'F', 'S'],
                numberOfRows: 5,
                clickEvents: {
                    click: function (target) {
                        var date, year, month, day;

                        date = target.date._i.split('-'),
                            year = date[0],
                            month = date[1],
                            day = date[2];

                        Backbone.history.navigate('#results/' + day + '/' + month + '/' + year, {trigger: true, silent: false});

                    },
                    onMonthChange: function (month) {
                        $clndr.find('.clndr-previous-button').text('<');
                        $clndr.find('.clndr-next-button').text('>');
                    }
                }
            });

            $clndr.find('.clndr-previous-button').text('<');
            $clndr.find('.clndr-next-button').text('>');


        },
        changeDay: function () {
            Backbone.history.navigate('#results', {trigger: true, silent: false});
        },
        addFavorites: function () {
            Backbone.history.navigate('#favorites', {trigger: true, silent: false});
        }
    });

    app.views.Result = Backbone.View.extend({
        initialize: function (options) {
            
            this.className = options.className || this.className;
            this.template = app.helpers.template('result', 'resulttpl', this.model.toJSON());
        },
        tagName: 'tr',
        className: '',
        render: function () {
            this.$el.html(this.template);
            return this;
        }
    });


}());
